<?php

namespace jrosset\CliProgram\Monolog;

use Monolog\Logger;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Provide a {@see LoggerInterface Monolog Logger} for {@see OutputInterface}
 */
trait TOutputInterfaceWithMonolog {
    /**
     * @var LoggerInterface|null The logger
     */
    private ?LoggerInterface $logger;

    /**
     * The logger
     *
     * @return LoggerInterface|null The logger
     */
    public function getLogger (): ?LoggerInterface {
        return $this->logger;
    }
    /**
     * Set the logger
     *
     * @param LoggerInterface|null $logger The logger
     *
     * @return $this
     */
    public function setLogger (?LoggerInterface $logger): self {
        $this->logger = $logger;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function write($messages, bool $newline = false, int $options = 0): void {
        if (!is_iterable($messages)) {
            $messages = [$messages];
        }

        if ($this->logger !== null && (($options & ConsoleOutputWithMonolog::OPTION_SKIP_MONOLOG) !== ConsoleOutputWithMonolog::OPTION_SKIP_MONOLOG)) {
            $verbosities = OutputInterface::VERBOSITY_QUIET | OutputInterface::VERBOSITY_NORMAL | OutputInterface::VERBOSITY_VERBOSE | OutputInterface::VERBOSITY_VERY_VERBOSE | OutputInterface::VERBOSITY_DEBUG;
            $verbosity = $verbosities & $options ?: OutputInterface::VERBOSITY_NORMAL;

            if ($verbosity <= OutputInterface::VERBOSITY_QUIET) {
                $loggerLevel = Logger::ERROR;
            }
            elseif ($verbosity <= OutputInterface::VERBOSITY_NORMAL) {
                $loggerLevel = Logger::NOTICE;
            }
            elseif ($verbosity <= OutputInterface::VERBOSITY_VERBOSE) {
                $loggerLevel = Logger::INFO;
            }
            elseif ($verbosity <= OutputInterface::VERBOSITY_VERY_VERBOSE) {
                $loggerLevel = Logger::INFO;
            }
            elseif ($verbosity <= OutputInterface::VERBOSITY_DEBUG) {
                $loggerLevel = Logger::DEBUG;
            }
            else {
                $loggerLevel = Logger::NOTICE;
            }

            foreach ($messages as $message) {
                $this->logger->log($loggerLevel, strip_tags($message));
            }
        }

        parent::write($messages, $newline, $options);
    }
}